package com.supermap.samplecode.realspace;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;

import com.supermap.data.*;
import com.supermap.realspace.Selection3D;
import com.supermap.realspace.StopArrivedEvent;
import com.supermap.realspace.StopArrivedListener;
import com.supermap.ui.*;
import com.supermap.ui.MapControl.Cursors;

/**
 * <p>
 * Title:三维符号
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：示范三维符号（三维模型符号、线符号）在场景中的显示与操作。 
 * 2、示例数据：
 *    安装目录\SampleData\Symbol3DDisplay\Symbol3DDisplay.smwu
 *    安装目录\SampleData\Symbol3DDisplay\Model\Subway.kml
 *    安装目录\SampleData\Symbol3DDisplay\flyground.fpf
 *    安装目录\SampleData\SymbolLibrary\MarkerLibrary3D.sym
 *    安装目录\SampleData\SymbolLibrary\LineLibrary3D.lsl 
 * 3、关键类型/成员:
 *    Workspace.getResources 方法
 *    Resources.getMarkerLibrary 方法
 *    Resources.getLineLibrary 方法
 *    SymbolMarkerLibrary.fromFile 方法
 *    SymbolLineLibrary.fromFile 方法
 *    SceneControl.getScene 方法
 *    SceneControl.MouseEvent 事件
 *    SceneControl.Tracked3DEvent 事件 
 *    Scene.getUnderground 方法
 *    Scene.pixelToGlobe 方法 
 *    DatasetVector.getRecordset 方法
 *    Recordset.addNew 方法 
 *    Recordset.update 方法
 *    GeoStyle3D.setMarkerSymbolID 方法 
 *    GeoStyle3D.setMarkerSizeFixed 方法
 *    GeoStyle3D.setMarkerSize 方法 
 *    GeoStyle3D.setMarker3DScaleX 方法 
 *    GeoStyle3D.setMarker3DScaleY 方法 
 *    GeoStyle3D.setMarker3DScaleZ 方法 
 *    GeoStyle3D.setLineSymbolID 方法 
 *    GeoStyle3D.setLineWidth 方法 
 *    GeoStyle3D.setAltitudeMode 方法 
 *    GeoStyle3D.setBottomAltitude 方法 
 *    GeoPoint3D.setStyle3D 方法
 *    GeoLine3D.setStyle3D 方法
 * 4、使用步骤：
 *   (1)在符号交互去，选择点符号类型，在场景中绘制点； 
 *   (2)选择线符号类型，在场景中绘制线，同时可执行沿所绘制线飞行的操作；
 *   (3)结束绘制操作后，将鼠标状态切换到漫游状态； 
 *   (4)在符号展示区选择北京地铁线，观看该线数据的动态飞行的效果;
 *   (5)在场景中选择一条线数据，可执行沿所绘制线飞行的操作。
 * ----------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 *Title: 3D Symbols
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Copyright Statement----------------------------
 * SuperMap Objects Java Sample Code 
 * Copyright: SuperMap Software Co., Ltd. All Rights Reserved.
 * ------------------------------------------------------------------------------
 *-----------------------Description--------------------------
 * 
 * 1. This sample shows you 3D symbols (points and lines) in the scene 
 * 2. Data:
 * \SampleData\Symbol3DDisplay\Symbol3DDisplay.smwu
 * \SampleData\Symbol3DDisplay\Model\Subway.kml
 * \SampleData\Symbol3DDisplay\flyground.fpf
 * \SampleData\SymbolLibrary\Library3D\Maker3DLibrary.sym
 * SampleData\SymbolLibrary\Library3D\Line3DLibrary.lsl 
 * 3. Key classes and members
 *    Workspace.getResources method
 *    Resources.getMarkerLibrary method
 *    Resources.getLineLibrary method
 *    SymbolMarkerLibrary.fromFile method
 *    SymbolLineLibrary.fromFile method
 *    SceneControl.getScene method
 *    SceneControl.MouseEvent event
 *    SceneControl.Tracked3DEvent event 
 *    Scene.getUnderground method

 *    Scene.pixelToGlobe method 
 *    DatasetVector.getRecordset method
 *    Recordset.addNew method 
 *    Recordset.update method
 *    GeoStyle3D.setMarkerSymbolID method 
 *    GeoStyle3D.setMarkerSizeFixed method
 *    GeoStyle3D.setMarkerSize method 
 *    GeoStyle3D.setMarker3DScaleX method 
 *    GeoStyle3D.setMarker3DScaleY method 
 *    GeoStyle3D.setMarker3DScaleZ method 
 *    GeoStyle3D.setLineSymbolID method 
 *    GeoStyle3D.setLineWidth method 
 *    GeoStyle3D.setAltitudeMode method 
 *    GeoStyle3D.setBottomAltitude method 
 *    GeoPoint3D.setStyle3D method
 *    GeoLine3D.setStyle3D method
 * 4. Steps:
 *   (1)In Draw Symbols, select a point symbol, and draw it in the scene 
 *   (2)Select a line symbol, and draw it in the scene
 *   (3)Switch the mouse status to Pan 
 *   (4)In Symbol Display, click Simulate Subway Line Running
 *   (5)Select a line in the scene, and click Fly along the line.

 * ----------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
public class MainFrame extends JFrame {

	private JLabel label;

	private static final long serialVersionUID = 1L;
	
	private SampleRun m_sampleRun;
	
	private Workspace m_workspace;
	
	private JPanel m_leftPanel;
	
	private JPanel m_drawPanel;
	
	private JPanel m_drawPointPanel;
	
	private JPanel m_drawLinePanel;
	
	private JPanel m_clearPanel;
	
	private JPanel m_displayPanel;
	
	private JPanel m_scenecontrolPanel;

	private JComboBox ComBoxLine3D;
	
	private JComboBox ComBoxPoint3D;

	private JButton ButtonClearPoint;
	
	private JButton ButtonFlyLine;
	
	private JButton ButtonFlyLineStop;
	
	private JButton ButtonClearLine;
	
	private JButton ButtonPan;
	
	private JButton ButtonFlyUndergroud;

	private Geometry3D geometry;
	
	private SceneControl m_sceneControl;

	private JPanel m_contentPane;

	private Selection3D[] selection;
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainFrame thisClass = new MainFrame();
				thisClass.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				thisClass.setVisible(true);
			}
		});

	}

	public MainFrame() {
		super();
		initialize();
		initializeCultureResources();

	}

	private void initialize() {
		this.setExtendedState(Frame.MAXIMIZED_BOTH);
		this.setSize(795, 480);
		this.setContentPane(getJContentPane());
		this.setTitle("三维符号");
		addWindowListener(new WindowAdapter() {
			/*
			 * 在主窗体加载时，初始化SampleRun类型，来完成功能的展现
			 * Initialize the SampleRun type when loading the main window
			 */
			public void windowOpened(final WindowEvent e) {
				// 实例化SampleRun
				// Initialize a new instance of SampleRun
				m_workspace = new Workspace();
				m_sampleRun = new SampleRun(m_workspace, m_sceneControl);
				ButtonFlyLine.setEnabled(false);
				m_sampleRun.flyManager.addStopArrivedListener(new StopArrivedListener() {
							public void stopArrived(StopArrivedEvent e) {
								try {
									if (e.getRouteStop().getName().equals("地铁上方1")) {
										ButtonFlyUndergroud.setText(getText("ButtonFlyUndergroud"));
										m_sampleRun.flyUndergroudStop();
										ComBoxLine3D.setEnabled(true);
										ComBoxPoint3D.setEnabled(true);
									}
									if (e.getRouteStop().getName().equals(m_sampleRun.routestopName)) {
										m_sampleRun.flyLineStop();
										selection[0].clear();
										ButtonFlyLineStop.setEnabled(false);
										ButtonFlyLine.setEnabled(false);
									}
								} catch (Exception ex) {
									System.out.printf(ex.getMessage());
								}
							}
						});
				ButtonClearLine.setEnabled(false);
				ButtonClearPoint.setEnabled(false);
				ButtonFlyLineStop.setEnabled(false);
			}

			/*
			 * 窗体关闭时清除所有数据集
			 * Clear all datasets when you close the form
			 */
			public void windowClosing(final WindowEvent e) {
				m_sampleRun.clearLineDataset();
				m_sampleRun.clearPointDataset();
				m_sceneControl.dispose();
				m_workspace.dispose();
			}
		});
	}

	private void initializeCultureResources()
	{
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			this.setTitle("三维符号");
		}
		else
		{
			this.setTitle("3D Symbols");
		}
	}
	
	private JPanel getJContentPane() {
		if (m_contentPane == null) {
			m_contentPane = new JPanel();
			m_contentPane.setLayout(new BorderLayout());
			m_contentPane.add(getLeftPanel(), BorderLayout.WEST);
			m_contentPane.add(getsceneControlPanel(), BorderLayout.CENTER);
		}
		return m_contentPane;
	}

	private JPanel getsceneControlPanel() {
		if (m_scenecontrolPanel == null) {
			m_scenecontrolPanel = new JPanel();
			BorderLayout borderLayout = new BorderLayout();
			borderLayout.setHgap(5);
			m_scenecontrolPanel.setLayout(borderLayout);
			m_scenecontrolPanel.add(getscenecontrol());
		}
		return m_scenecontrolPanel;
	}

	private SceneControl getscenecontrol() {
		if (m_sceneControl == null) {
			m_sceneControl = new SceneControl();
			// 添加选中对象事件
			// Add the selected object event
			m_sceneControl.addObjectSelectedListener(new ObjectSelectedListener() {
						public void objectSelected(final ObjectSelectedEvent e) {
							if (e.getCount() > 0) {
								selection = m_sceneControl.getScene().findSelection(true);
								Recordset recordeset = selection[0].toRecordset();
								selection[0].toRecordset();
								geometry = (Geometry3D) recordeset.getGeometry();
								if (geometry.getType() == GeometryType.GEOLINE3D) {
									ButtonFlyLine.setEnabled(true);
								} else {
									ButtonFlyLineStop.setEnabled(false);
									ButtonFlyLine.setEnabled(false);
								}
							}
						}
					});

		}
		return m_sceneControl;
	}

	private JPanel getLeftPanel() {
		if (m_leftPanel == null) {
			m_leftPanel = new JPanel();
			final GridBagLayout gridBagLayout = new GridBagLayout();
			gridBagLayout.rowHeights = new int[] { 0, 7 };
			m_leftPanel.setLayout(gridBagLayout);
			final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
			gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints_1.anchor = GridBagConstraints.NORTH;
			gridBagConstraints_1.insets = new Insets(0, 0, 0, 0);
			gridBagConstraints_1.weighty = 1;
			gridBagConstraints_1.gridx = 0;
			gridBagConstraints_1.gridy = 1;
			m_leftPanel.add(getDrawPanel(), gridBagConstraints_1);
			final GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = GridBagConstraints.BOTH;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;
			gridBagConstraints.weighty = 0.02;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.ipadx = 45;
			m_leftPanel.add(getDisplayPanel(), gridBagConstraints);
		}
		return m_leftPanel;

	}

	private JPanel getDisplayPanel() {
		if (m_displayPanel == null) {
			m_displayPanel = new JPanel();
			m_displayPanel.setBorder(new TitledBorder(null, getText("m_displayPanel"),
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null));
			m_displayPanel.add(getButtonFlyUndergroud());
		}
		return m_displayPanel;
	}

	/*
	 * 地铁八号线动态飞行
	 * Simulate Subway Line Running
	 */
	private JButton getButtonFlyUndergroud() {
		if (ButtonFlyUndergroud == null) {
			ButtonFlyUndergroud = new JButton();
			ButtonFlyUndergroud.setMargin(new Insets(0, 0, 0, 0));
			ButtonFlyUndergroud.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {

					m_sceneControl.setCursorCustomized(false);
					m_sceneControl.setAction(Action3D.PAN2);

					if (ButtonFlyUndergroud.getText() == getText("ButtonFlyUndergroud")) {
						if (m_sceneControl.getScene().getLayers().getCount() != 0) {
							m_sceneControl.getScene().getLayers().clear();
						}
						m_sceneControl.getScene().getUnderground().setVisible(
								true);
						ButtonFlyUndergroud.setText(getText("ButtonFlyUndergroud_Stop"));
						m_sampleRun.flyUndergroud();
						ButtonFlyLine.setEnabled(false);
						ComBoxLine3D.setEnabled(false);
						ComBoxPoint3D.setEnabled(false);
						ButtonClearLine.setEnabled(false);
						ButtonClearPoint.setEnabled(false);
					} else {
						ButtonFlyUndergroud.setText(getText("ButtonFlyUndergroud"));
						m_sampleRun.flyUndergroudStop();
						ComBoxLine3D.setEnabled(true);
						ComBoxPoint3D.setEnabled(true);
					}
				}
			});
			ButtonFlyUndergroud.setText(getText("ButtonFlyUndergroud"));
		}
		return ButtonFlyUndergroud;
	}

	/*
	 * 添加交互区Panel
	 * Add Panel
	 */
	private JPanel getDrawPanel() {
		if (m_drawPanel == null) {
			m_drawPanel = new JPanel();
			m_drawPanel.setLayout(new GridBagLayout());
			m_drawPanel.setBorder(new TitledBorder(null, getText("m_drawPanel"),
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null));
			final GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.insets = new Insets(0, 0, 0, 0);
			m_drawPanel.add(getDrawPointPanel(), gridBagConstraints);
			final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
			gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints_1.anchor = GridBagConstraints.NORTH;
			gridBagConstraints_1.weighty = 1.0;
			gridBagConstraints_1.gridy = 2;
			gridBagConstraints_1.gridx = 0;
			m_drawPanel.add(getclearLinePanel(), gridBagConstraints_1);
			final GridBagConstraints gridBagConstraints_2 = new GridBagConstraints();
			gridBagConstraints_2.fill = GridBagConstraints.BOTH;
			gridBagConstraints_2.gridy = 1;
			gridBagConstraints_2.gridx = 0;
			m_drawPanel.add(getDrawLinePanel(), gridBagConstraints_2);
		}
		return m_drawPanel;
	}

	private JPanel getclearLinePanel() {
		if (m_clearPanel == null) {
			m_clearPanel = new JPanel();
			m_clearPanel.setLayout(new GridBagLayout());
			m_clearPanel.setBorder(new TitledBorder(null, getText("m_clearPanel"),
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null));

			final GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridwidth = 2;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.insets = new Insets(0, 0, 5, 0);
			m_clearPanel.add(getButtonPan(), gridBagConstraints);
			
			final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
			gridBagConstraints_1.fill = GridBagConstraints.BOTH;
			gridBagConstraints_1.gridx = 0;
			gridBagConstraints_1.gridy = 2;
			gridBagConstraints_1.ipadx = 90;
			gridBagConstraints_1.ipady = 25;
			gridBagConstraints_1.insets = new Insets(0, 0, 5, 5);
			final GridBagConstraints gridBagConstraints_3 = new GridBagConstraints();
			gridBagConstraints_3.insets = new Insets(0, 0, 5, 0);
			gridBagConstraints_3.ipadx = 30;
			gridBagConstraints_3.anchor = GridBagConstraints.WEST;
			gridBagConstraints_3.gridwidth = 2;
			gridBagConstraints_3.gridy = 1;
			gridBagConstraints_3.gridx = 0;
			m_clearPanel.add(getLabel(), gridBagConstraints_3);
			m_clearPanel.add(getButtonFlyLine(), gridBagConstraints_1);
			
			final GridBagConstraints gridBagConstraints_2 = new GridBagConstraints();
			gridBagConstraints_2.gridx = 1;
			gridBagConstraints_2.gridy = 2;
			gridBagConstraints_2.ipadx = 50;
			gridBagConstraints_2.ipady = 28;
			gridBagConstraints_2.insets = new Insets(0, 5, 5, 0);
			m_clearPanel.add(getButtonFlyLineStop(), gridBagConstraints_2);
		}

		return m_clearPanel;
	}

	/*
	 * 切换到漫游状态
	 * Switch the mouse status to Pan
	 */
	private JButton getButtonPan() {
		if (ButtonPan == null) {
			ButtonPan = new JButton();
			ButtonPan.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {
					m_sceneControl.setCursorCustomized(false);
					m_sceneControl.setAction(Action3D.PAN);
					m_sampleRun.flag = true;
					m_sampleRun.isDraw = false;
				}
			});
			ButtonPan.setText(getText("ButtonPan"));
		}
		return ButtonPan;
	}

	private JPanel getDrawLinePanel() {
		if (m_drawLinePanel == null) {
			m_drawLinePanel = new JPanel();
			m_drawLinePanel.setLayout(new GridBagLayout());
			m_drawLinePanel.setBorder(new TitledBorder(null, getText("m_drawLinePanel"),
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null));
			final GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.insets = new Insets(5, 0, 5, 0);
			m_drawLinePanel.add(getComBoxLine3D(), gridBagConstraints);
			final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
			gridBagConstraints_1.anchor = GridBagConstraints.EAST;
			gridBagConstraints_1.gridx = 0;
			gridBagConstraints_1.gridy = 1;
			gridBagConstraints_1.insets = new Insets(0, 5, 5, 0);
			m_drawLinePanel.add(getbuttonClearLine(), gridBagConstraints_1);
		}
		return m_drawLinePanel;
	}

	/*
	 * 清除线操作
	 * Clear lines
	 */
	private JButton getbuttonClearLine() {
		if (ButtonClearLine == null) {

			ButtonClearLine = new JButton();
			ButtonClearLine.setName("buttonClearLine");
			ButtonClearLine.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {
					m_sampleRun.clearLineDataset();
					ComBoxLine3D.setSelectedIndex(0);
					m_sceneControl.setCursorCustomized(false);
					m_sceneControl.setAction(Action3D.PAN2);
					ButtonClearLine.setEnabled(false);
				}
			});
			ButtonClearLine.setText(getText("ButtonClearLine"));
		}
		return ButtonClearLine;
	}

	/*
	 * 沿线飞行停止
	 * Stop flying
	 */
	private JButton getButtonFlyLineStop() {
		if (ButtonFlyLineStop == null) {
			ButtonFlyLineStop = new JButton();
			ButtonFlyLineStop.setName("buttonFlyLineStop");
			ButtonFlyLineStop.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {
					m_sampleRun.flyLineStop();
				}
			});
			ButtonFlyLineStop.setPreferredSize(new Dimension(10, 0));
			ButtonFlyLineStop.setText(getText("ButtonFlyLineStop"));
		}
		return ButtonFlyLineStop;
	}

	/*
	 * 沿线飞行
	 * Fly along the line
	 */
	private JButton getButtonFlyLine() {
		if (ButtonFlyLine == null) {
			ButtonFlyLine = new JButton();
			ButtonFlyLine.setName("buttonFlyLine");
			ButtonFlyLine.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {
					m_sceneControl.getScene().setCrossVisible(true);
					m_sampleRun.flyLine(geometry);
					ButtonFlyLineStop.setEnabled(true);
				}
			});
			ButtonFlyLine.setToolTipText("");
			ButtonFlyLine.setVerticalTextPosition(SwingConstants.CENTER);
			ButtonFlyLine.setMaximumSize(new Dimension(0, 0));
			ButtonFlyLine.setIconTextGap(0);
			ButtonFlyLine.setPreferredSize(new Dimension(0, 0));
			ButtonFlyLine.setText(getText("ButtonFlyLine"));
		}
		return ButtonFlyLine;
	}

	private JComboBox getComBoxLine3D() {
		if (ComBoxLine3D == null) {
			ComBoxLine3D = new JComboBox();
			ComBoxLine3D.setName("comBoxLine3D");
			ComBoxLine3D.addItemListener(new ItemListener() {
				public void itemStateChanged(final ItemEvent e) {
					m_sampleRun.settingLine3D(ComBoxLine3D.getSelectedIndex());
					ButtonClearLine.setEnabled(true);
				}
			});
			ComBoxLine3D.setModel(new DefaultComboBoxModel(new String[] {
					getText("m_Selectline"), getText("Railway"), getText("Pipeline"),getText("Road") }));
		}
		return ComBoxLine3D;
	}

	/*
	 * 绘制点Panel
	 * Draw Point Panel
	 */
	private JPanel getDrawPointPanel() {
		if (m_drawPointPanel == null) {
			m_drawPointPanel = new JPanel();
			m_drawPointPanel.setLayout(new GridBagLayout());
			m_drawPointPanel.setAutoscrolls(false);
			m_drawPointPanel.setBorder(new TitledBorder(null, getText("m_drawPointPanel"),
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, null, null));
			final GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.weighty = 0.2;
			gridBagConstraints.gridx = 0;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.ipadx = 5;
			gridBagConstraints.ipady = 1;
			gridBagConstraints.insets = new Insets(5, 5, 5, 0);
			m_drawPointPanel.add(getComboxpoint3D(), gridBagConstraints);
			final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
			gridBagConstraints_1.anchor = GridBagConstraints.EAST;
			gridBagConstraints_1.gridx = 0;
			gridBagConstraints_1.gridy = 1;
			gridBagConstraints_1.insets = new Insets(0, 0, 5, 0);
			m_drawPointPanel.add(getButtonClearPoint(), gridBagConstraints_1);
		}
		return m_drawPointPanel;
	}

	/*
	 * 添加Button清除点操作
	 * Clear Points
	 */
	private JButton getButtonClearPoint() {
		if (ButtonClearPoint == null) {
			ButtonClearPoint = new JButton();
			ButtonClearPoint.setRolloverEnabled(false);
			ButtonClearPoint.setRequestFocusEnabled(true);
			ButtonClearPoint.addActionListener(new ActionListener() {
				public void actionPerformed(final ActionEvent e) {
					m_sampleRun.clearPointDataset();
					ComBoxPoint3D.setSelectedIndex(0);
					m_sceneControl.setCursorCustomized(false);
					m_sceneControl.setAction(Action3D.PAN2);
					ButtonClearPoint.setEnabled(false);
				}
			});
			ButtonClearPoint.setMargin(new Insets(2, 14, 2, 14));
			ButtonClearPoint.setText(getText("ButtonClearPoint"));
		}

		return ButtonClearPoint;
	}

	/*
	 * 添加COMBOX绘制点
	 * Add Combox to draw points
	 */
	private JComboBox getComboxpoint3D() {
		if (ComBoxPoint3D == null) {
			ComBoxPoint3D = new JComboBox();
			ComBoxPoint3D.addItemListener(new ItemListener() {
				public void itemStateChanged(final ItemEvent e) {
					m_sceneControl.setAction(Action3D.PAN2);
					m_sceneControl.setCursorCustomized(true);
					m_sceneControl.setCursor(Cursors.getCross());
					m_sampleRun.settingPoint3D(ComBoxPoint3D.getSelectedIndex());
					ButtonClearPoint.setEnabled(true);
				}
			});
			ComBoxPoint3D.setName("");
			ComBoxPoint3D.setMaximumRowCount(8);
			ComBoxPoint3D.setModel(new DefaultComboBoxModel(new String[] {
					getText("m_Selectpoint"), getText("Bus Stop"),getText("Street Light"), 
					getText("Tree"), getText("Car"), getText("Grass"),
					getText("Trash Bin")}));
		}
		return ComBoxPoint3D;
	}
	
	protected JLabel getLabel() {
		if (label == null) {
			label = new JLabel();
			label.setText(getText("label"));
		}
		return label;
	}

	private String getText(String id)
	{
		String text = "";
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			if(id == "ButtonFlyUndergroud")
			{
				text = "地铁八号线动态飞行";
			}
			if(id == "m_displayPanel")
			{
				text = "符号展示区";
			}
			if(id == "ButtonFlyUndergroud_Stop")
			{
				text = "停止动态飞行";
			}
			if(id == "m_drawPanel")
			{
				text = "符号交互区";
			}
			if(id == "m_clearPanel")
			{
				text = "鼠标状态切换";
			}
			if(id == "ButtonPan")
			{
				text = "切换到漫游状态";
			}
			if(id == "m_drawLinePanel")
			{
				text = "绘制线";
			}
			if(id == "ButtonClearLine")
			{
				text = "清除线操作";
			}
			if(id == "ButtonFlyLineStop")
			{
				text = "停止";
			}
			if(id == "ButtonFlyLine")
			{
				text = "沿线飞行";
			}
			if(id == "m_Selectline")
			{
				text = "请选择三维线符号类型";
			}
			if(id == "Railway")
			{
				text = "铁路";
			}
			if(id == "Pipeline")
			{
				text = "圆管";
			}
			if(id == "Road")
			{
				text = "公路";
			}
			if(id == "m_drawPointPanel")
			{
				text = "绘制点";
			}
			if(id == "ButtonClearPoint")
			{
				text = "清除点操作";
			}
			if(id == "m_Selectpoint")
			{
				text = "请选择三维点符号类型";
			}
			if(id == "Bus Stop")
			{
				text = "站牌";
			}
			if(id == "Street Light")
			{
				text = "路灯";
			}
			if(id == "Tree")
			{
				text = "树木";
			}
			if(id == "Car")
			{
				text = "汽车";
			}
			if(id == "Grass")
			{
				text = "花草";
			}
			if(id == "Trash Bin")
			{
				text = "垃圾桶";
			}
			if(id == "label")
			{
				text = "请在场景中选择一条线";
			}
		}
		else
		{
			if(id == "ButtonFlyUndergroud")
			{
				text = "Simulate Subway Line Running";
			}
			if(id == "m_displayPanel")
			{
				text = "Symbol Display";
			}
			if(id == "ButtonFlyUndergroud_Stop")
			{
				text = "Stop Flying";
			}
			if(id == "m_drawPanel")
			{
				text = "Draw Symbols";
			}
			if(id == "m_clearPanel")
			{
				text = "Switch Mouse Status";
			}
			if(id == "ButtonPan")
			{
				text = "Switch to Pan";
			}
			if(id == "m_drawLinePanel")
			{
				text = "Draw Line";
			}
			if(id == "ButtonClearLine")
			{
				text = "Clear Lines";
			}
			if(id == "ButtonFlyLineStop")
			{
				text = "Stop";
			}
			if(id == "ButtonFlyLine")
			{
				text = "Fly";
			}
			if(id == "m_Selectline")
			{
				text = "Select a line symbol";
			}
			if(id == "Railway")
			{
				text = "Railway";
			}
			if(id == "Pipeline")
			{
				text = "Pipeline";
			}
			if(id == "Road")
			{
				text = "Road";
			}
			if(id == "m_drawPointPanel")
			{
				text = "Draw Point";
			}
			if(id == "ButtonClearPoint")
			{
				text = "Clear Points";
			}
			if(id == "m_Selectpoint")
			{
				text = "Select a point symbol";
			}
			if(id == "Bus Stop")
			{
				text = "Bus Stop";
			}
			if(id == "Street Light")
			{
				text = "Street Light";
			}
			if(id == "Tree")
			{
				text = "Tree";
			}
			if(id == "Car")
			{
				text = "Car";
			}
			if(id == "Grass")
			{
				text = "Grass";
			}
			if(id == "Trash Bin")
			{
				text = "Trash Bin";
			}
			if(id == "label")
			{
				text = "Select a line";
			}
		}
		return text;
	}
}

