package com.supermap.samplecode.realspace;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.JTextField;
import com.supermap.data.*;
import com.supermap.ui.*;
import com.supermap.realspace.*;

/**
 * <p>
 * Title:三维符号
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：示范三维符号（三维模型符号、线符号）在场景中的显示与操作。 
 * 2、示例数据：
 *    安装目录\SampleData\Symbol3DDisplay\Symbol3DDisplay.smwu
 *    安装目录\SampleData\Symbol3DDisplay\Model\Subway.kml
 *    安装目录\SampleData\Symbol3DDisplay\flyground.fpf
 *    安装目录\SampleData\SymbolLibrary\MarkerLibrary3D.sym
 *    安装目录\SampleData\SymbolLibrary\LineLibrary3D.lsl 
 * 3、关键类型/成员:
 *    Workspace.getResources 方法
 *    Resources.getMarkerLibrary 方法
 *    Resources.getLineLibrary 方法
 *    SymbolMarkerLibrary.fromFile 方法
 *    SymbolLineLibrary.fromFile 方法
 *    SceneControl.getScene 方法
 *    SceneControl.MouseEvent 事件
 *    SceneControl.Tracked3DEvent 事件 
 *    Scene.getUnderground 方法
 *    Scene.pixelToGlobe 方法 
 *    DatasetVector.getRecordset 方法
 *    Recordset.addNew 方法 
 *    Recordset.update 方法
 *    GeoStyle3D.setMarkerSymbolID 方法 
 *    GeoStyle3D.setMarkerSizeFixed 方法
 *    GeoStyle3D.setMarkerSize 方法 
 *    GeoStyle3D.setMarker3DScaleX 方法 
 *    GeoStyle3D.setMarker3DScaleY 方法 
 *    GeoStyle3D.setMarker3DScaleZ 方法 
 *    GeoStyle3D.setLineSymbolID 方法 
 *    GeoStyle3D.setLineWidth 方法 
 *    GeoStyle3D.setAltitudeMode 方法 
 *    GeoStyle3D.setBottomAltitude 方法 
 *    GeoPoint3D.setStyle3D 方法
 *    GeoLine3D.setStyle3D 方法
 * 4、使用步骤：
 *   (1)在符号交互去，选择点符号类型，在场景中绘制点； 
 *   (2)选择线符号类型，在场景中绘制线，同时可执行沿所绘制线飞行的操作；
 *   (3)结束绘制操作后，将鼠标状态切换到漫游状态； 
 *   (4)在符号展示区选择北京地铁线，观看该线数据的动态飞行的效果;
 *   (5)在场景中选择一条线数据，可执行沿所绘制线飞行的操作。
 * ----------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 *Title: 3D Symbols
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Copyright Statement----------------------------
 * SuperMap Objects Java Sample Code 
 * Copyright: SuperMap Software Co., Ltd. All Rights Reserved.
 * ------------------------------------------------------------------------------
 *-----------------------Description--------------------------
 * 
 * 1. This sample shows you 3D symbols (points and lines) in the scene 
 * 2. Data:
 * \SampleData\Symbol3DDisplay\Symbol3DDisplay.smwu
 * \SampleData\Symbol3DDisplay\Model\Subway.kml
 * \SampleData\Symbol3DDisplay\flyground.fpf
 * \SampleData\SymbolLibrary\Library3D\Maker3DLibrary.sym
 * SampleData\SymbolLibrary\Library3D\Line3DLibrary.lsl 
 * 3. Key classes and members
 *    Workspace.getResources method
 *    Resources.getMarkerLibrary method
 *    Resources.getLineLibrary method
 *    SymbolMarkerLibrary.fromFile method
 *    SymbolLineLibrary.fromFile method
 *    SceneControl.getScene method
 *    SceneControl.MouseEvent event
 *    SceneControl.Tracked3DEvent event 
 *    Scene.getUnderground method

 *    Scene.pixelToGlobe method 
 *    DatasetVector.getRecordset method
 *    Recordset.addNew method 
 *    Recordset.update method
 *    GeoStyle3D.setMarkerSymbolID method 
 *    GeoStyle3D.setMarkerSizeFixed method
 *    GeoStyle3D.setMarkerSize method 
 *    GeoStyle3D.setMarker3DScaleX method 
 *    GeoStyle3D.setMarker3DScaleY method 
 *    GeoStyle3D.setMarker3DScaleZ method 
 *    GeoStyle3D.setLineSymbolID method 
 *    GeoStyle3D.setLineWidth method 
 *    GeoStyle3D.setAltitudeMode method 
 *    GeoStyle3D.setBottomAltitude method 
 *    GeoPoint3D.setStyle3D method
 *    GeoLine3D.setStyle3D method
 * 4. Steps:
 *   (1)In Draw Symbols, select a point symbol, and draw it in the scene 
 *   (2)Select a line symbol, and draw it in the scene
 *   (3)Switch the mouse status to Pan 
 *   (4)In Symbol Display, click Simulate Subway Line Running
 *   (5)Select a line in the scene, and click Fly along the line.

 * ----------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 * 
 */
class NetworkGeometry {
	public Geometry geometry;
	public int nID;
	public int nFNode;
	public int nTNode;
	public boolean bIsIn;

	public NetworkGeometry() {
		nID = -1;
		nFNode = -1;
		nTNode = -1;
		geometry = null;
		bIsIn = true;
	}
}

public class SampleRun {

	private SceneControl m_sceneControl;
	private Workspace m_workspace;
	public FlyManager flyManager;
	private ArrayList<NetworkGeometry> m_SelectedGeoNetwork;
	private double RTOD = 57.295779513082320876798154814;
	private double DTOR = 0.017453292519943295769236907684894;
	public Boolean isDraw = false;
	// flag = false时,表示画点, flag = true时,表示画线
	// flag = false means drawing points; flag = true means drawing lines
	public Boolean flag = true; 
	private Boolean PointLineCamera = true;
	public Route route;
	public int routestopCount;
	private RouteStop stop;
	public String routestopName;
	
    private int m_line3DIndex;
    private int m_marker3DIndex;
    
    private DatasetVector m_datasetLine3D;
    private DatasetVector m_datasetPoint3D;
    
    private Layer3DDataset m_layer3DPoint;
    private Layer3DDataset m_layer3DLine;

	private MouseAdapter m_mouseClick = new MouseAdapter() {
		public void mouseClicked(final MouseEvent e) {
			if (e.getClickCount() == 1) {
				try {
					if (e.getButton() == 1 && isDraw) {
						if (!flag) {
							Point3D pt3d = new Point3D();
							
							pt3d = m_sceneControl.getScene().pixelToGlobe(new Point(e.getX(),e.getY()), PixelToGlobeMode.TERRAINANDMODEL);
							GeoPoint3D geopoint3D = new GeoPoint3D(pt3d);
							
							Recordset recordset = m_datasetPoint3D.getRecordset(
									false, CursorType.DYNAMIC);
							recordset.moveLast();							
							
							int ID = 0;
							switch (m_marker3DIndex) {
							case 1: {
								ID = 253152;
								break;
							}
							case 2: {
								ID = 253164;
								break;
							}
							case 3: {
								ID = 253301;
								break;
							}
							case 4: {
								ID = 253265;
								break;
							}
							case 5: {
								ID = 253177;
								break;
							}
							case 6: {
								ID = 253155;
								break;
								}
							}

							GeoStyle3D geoStyle = new GeoStyle3D();
							
							geoStyle.setMarkerSymbolID(ID);
							geoStyle.setMarkerSizeFixed(false);
							geoStyle.setMarkerSize(1);
							geoStyle.setMarker3DScaleX(1);
							geoStyle.setMarker3DScaleY(1);
							geoStyle.setMarker3DScaleZ(1);
							geoStyle.setMarker3D(true);
							geoStyle.setAltitudeMode(AltitudeMode.RELATIVE_TO_GROUND);

							geopoint3D.setStyle3D(geoStyle);

							recordset.addNew(geopoint3D);
							recordset.update();
							
							m_layer3DPoint.setSelectable(false);
							m_layer3DPoint.updateData();
							m_sceneControl.getScene().refresh();
						}
					}
				} catch (Exception ex) {
					System.out.println(ex.getMessage());
				}
			}
		}
	};

	private Tracked3DListener m_tracked3DListener = new Tracked3DListener() {
		public void tracked(final Tracked3DEvent e) {
			try {
				int ID = 0;
				double lineWidth = 0.0;
				double bottomheight = 0;
				if (flag) {
					GeoLine3D geoLine3D = (GeoLine3D) e.getGeometry();					
					GeoStyle3D geoStyle = new GeoStyle3D();
					
					if(geoLine3D == null){
						return;
					}
					switch (m_line3DIndex) {
						case 1: {
							ID = 962046;
							lineWidth = 100;
							bottomheight = 5;
							break;
						}
		
						case 2: {
							ID = 962048;
							lineWidth = 0.35;
							bottomheight = -15;
							break;
						}
						case 3: {
							ID = 962051;
							lineWidth = 25;
							bottomheight = 11;
							break;
						}
					}
					
					geoStyle.setLineSymbolID(ID);
					geoStyle.setLineWidth(lineWidth);
					geoStyle.setAltitudeMode(AltitudeMode.RELATIVE_TO_GROUND);
					//geoStyle.setBottomAltitude(bottomheight);
					
					geoLine3D.setStyle3D(geoStyle);				

					Recordset recordset = m_datasetLine3D.getRecordset(false,CursorType.DYNAMIC);
					recordset.addNew(geoLine3D);
					recordset.update();
					recordset.dispose();
					
					m_layer3DLine.updateData();
					m_sceneControl.getScene().refresh();					
				}
			}				
			catch (Exception ex) {
				System.out.printf(ex.getMessage());
			}
		}
	};

	/*
	 * 根据sceneControl构造 SampleRun对象
	 * Initialize the SampleRun object with the specified sceneControl
	 */
	public SampleRun(Workspace workspace, SceneControl sceneControl) {

		try {
			this.m_sceneControl = sceneControl;
			this.m_workspace = workspace;
			m_sceneControl.getScene().setWorkspace(m_workspace);
			m_sceneControl.getScene().setCrossVisible(false);
			flyManager = new FlyManager();
			flyManager.setScene(m_sceneControl.getScene());
			initialize();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	private void initialize() {
		loadDataset();
		comBoxDisplayLine3D(1);
		clearLineDataset();
		clearPointDataset();
	}

	/*
	 * 绘制三维点符号
	 * Draw 3D point symbols
	 */
	public void settingPoint3D(int selectedIndex) {
		try {
			isDraw = true;
			m_marker3DIndex = selectedIndex;
			flag = false;
			if (!m_sceneControl.getScene().getLayers().contains("Ground@OlympicGreen")) {
				m_sceneControl.getScene().getLayers().clear();
				loadOlympicGreen();
				m_sceneControl.getScene().getUnderground().setVisible(false);
				Camera camera = m_sceneControl.getScene().getCamera();
				camera.setAltitude(22.0964799495414);
				camera.setHeading(15.9938386601762);
				camera.setLatitude(39.991413117045);
				camera.setLongitude(116.390531219105);
				camera.setTilt(86.913152133403);
				m_sceneControl.getScene().fly(camera, 1);
			}
			if (PointLineCamera) {
				Camera camera = m_sceneControl.getScene().getCamera();
				camera.setAltitude(22.0964799495414);
				camera.setHeading(15.9938386601762);
				camera.setLatitude(39.991413117045);
				camera.setLongitude(116.390531219105);
				camera.setTilt(86.913152133403);
				m_sceneControl.getScene().fly(camera, 1);
				PointLineCamera = false;
			}
			// 新建数据集
			// Create a dataset
			if (!m_workspace.getDatasources().get(0).getDatasets().contains(
					"Point3D")) {
				// 定义新数据集
				// Define a new dataset
				DatasetVectorInfo Datasetinfo = new DatasetVectorInfo();
				Datasetinfo.setType(DatasetType.CAD);
				Datasetinfo.setName("Point3D");
				DatasetVector datset = m_workspace.getDatasources().get(0)
						.getDatasets().create(Datasetinfo);
				
				PrjCoordSys prjcoord = new PrjCoordSys();
				prjcoord.setType(PrjCoordSysType.PCS_EARTH_LONGITUDE_LATITUDE);
				datset.setPrjCoordSys(prjcoord);
			}

			m_datasetPoint3D = (DatasetVector)m_workspace.getDatasources().get(0).getDatasets().get("Point3D");
			
			if (m_sceneControl.getScene().getLayers().contains("stylePoint3D")){
                m_layer3DPoint = (Layer3DDataset)m_sceneControl.getScene().getLayers().get("stylePoint3D");
            }
            else{
                m_layer3DPoint = m_sceneControl.getScene().getLayers().add(m_datasetPoint3D, new Layer3DSettingVector(), true, "stylePoint3D");
            }
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 清除点所有操作
	 * Clear points
	 */
	public void clearPointDataset() {
		try {
			m_sceneControl.getScene().getLayers().remove("stylePoint3D");
			m_sceneControl.getScene().refresh();
			m_workspace.getDatasources().get(0).getDatasets().delete("Point3D");
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 绘制三维线符号
	 * Draw 3D line symbols
	 */
	public void settingLine3D(int selectedIndex) {
		try {

			flag = true;
			m_line3DIndex = selectedIndex;
			switch (m_line3DIndex) {
				case 1: {
					if (!m_sceneControl.getScene().getLayers().contains("ditie3")) {
						comBoxDisplayLine3D(1);
					}
					break;
				}
				case 2: {
					if (!m_sceneControl.getScene().getLayers().contains(
							"underPipeLine3D")) {
						comBoxDisplayLine3D(2);
					}
					break;
				}
				case 3: {
					m_sceneControl.getScene().getUnderground().setVisible(false);
					if (!m_sceneControl.getScene().getLayers().contains(
							"Ground@OlympicGreen")) {
	
						m_sceneControl.getScene().getLayers().clear();
						loadOlympicGreen();
						Camera camera = new Camera(116.3842976377199,
								39.986785637662514, 341.96175131294876);
						camera.setTilt(63.988040332277215);
						camera.setHeading(4.18128919038237);
						m_sceneControl.getScene().fly(camera, 1);
						PointLineCamera = true;
					}
					if (!PointLineCamera) {
						Camera camera = new Camera(116.3842976377199,
								39.986785637662514, 341.96175131294876);
						camera.setTilt(63.988040332277215);
						camera.setHeading(4.18128919038237);
						m_sceneControl.getScene().fly(camera, 1);
						PointLineCamera = true;
					}
					break;
				}
			}
			m_sceneControl.setAction(Action3D.MeasureDistance);

			// 新建数据集
			// Create a dataset
			if (!m_workspace.getDatasources().get(0).getDatasets().contains(
					"Line3D")) {
				// 定义新数据集
				// Define a new dataset
				DatasetVectorInfo Datasetinfo = new DatasetVectorInfo();
				Datasetinfo.setType(DatasetType.CAD);
				Datasetinfo.setName("Line3D");
				DatasetVector datset = m_workspace.getDatasources().get(0).getDatasets().create(
						Datasetinfo);
				
				PrjCoordSys prjcoord = new PrjCoordSys();
                prjcoord.setType(PrjCoordSysType.PCS_EARTH_LONGITUDE_LATITUDE);
                datset.setPrjCoordSys(prjcoord);	
				
			}
			
			m_datasetLine3D = (DatasetVector)m_workspace.getDatasources().get(0).getDatasets().get("Line3D");
			
			if(m_sceneControl.getScene().getLayers().contains("styleLine3D"))
			{
				m_layer3DLine = (Layer3DDataset)m_sceneControl.getScene().getLayers().get("styleLine3D");
			}
			else
			{
				m_layer3DLine = m_sceneControl.getScene().getLayers().add(m_datasetLine3D, new Layer3DSettingVector(), true,"styleLine3D");
			}
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 沿线飞行
	 * Fly along the line
	 */
	public void flyLine(Geometry3D geometry) {
		try {
			GeoLine3D geoline = (GeoLine3D) geometry;
			route = new Route();
			route.fromGeoLine3D(geoline);
			route.setFlyAlongTheRoute(true);
			routestopCount = route.getStops().getCount();
			int i;
			for (i = 0; i < routestopCount; i++) {
				stop = route.getStops().get(i);
				Camera camera = stop.getCamera();
				camera.setTilt(50);
				camera.setAltitude(100);
				stop.setCamera(camera);
			}
			route.setTiltFixed(true);
			route.setHeadingFixed(true);
			route.setAltitudeFixed(true);
			route.setSpeed(110);
			route.setLinesVisible(false);
			route.setStopsVisible(false);
			flyManager.getRoutes().add(route);
			routestopName = route.getStops().get(routestopCount - 1).getName();
			flyManager.setScene(m_sceneControl.getScene());

			// 设置当前路线
			// Set the current path
			if (flyManager.getRoutes().getCount() == 1) {
				flyManager.getRoutes().setCurrentRouteIndex(0);
			} else {
				flyManager.getRoutes().setCurrentRouteIndex(1);
			}

			flyManager.play();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 沿线飞行停止
	 * Stop flying
	 */
	public void flyLineStop() {
		try {
			flyManager.stop();
			flyManager.getRoutes().remove(0);
			m_sceneControl.getScene().setCrossVisible(false);

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 清除线操作
	 * Clear lines
	 */
	public void clearLineDataset() {
		try {
			m_sceneControl.getScene().getLayers().remove("styleLine3D");
			m_sceneControl.getScene().refresh();
			m_workspace.getDatasources().get(0).getDatasets().delete("Line3D");
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 选择三维线符号进行展示
	 * Select 3D line symbols to display
	 */
	public void comBoxDisplayLine3D(int selectedIndex) {
		try {
			int selctionID = selectedIndex;
			// 先进行地表挖方,显示出地下三维线
			// Fill and cut to display the underground 3D lines
			m_sceneControl.getScene().getLayers().clear();
			m_sceneControl.getScene().refresh();
			m_sceneControl.getScene().getUnderground().setVisible(true);
			m_sceneControl.getScene().getGlobalImage().setTransparency(0);
			DatasetVector dataSet_Region = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("Region");

			int[] id = { 1 };
			Recordset recordset = dataSet_Region.query(id, CursorType.STATIC);
			GeoRegion georegion = (GeoRegion) recordset.getGeometry();
			 m_sceneControl.getScene().getGlobalImage().addExcavationRegion(georegion, "地表局部透明");

			switch (selctionID) {
			case 1: {
				loadSubway();
				break;
			}
			case 2: {
				loadPipeLine3D();
				break;
				}
			}

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 加载地下管道线
	 * Add underground pipelines
	 */
	private void loadPipeLine3D() {
		try {
			m_SelectedGeoNetwork = new ArrayList<NetworkGeometry>();
			DatasetVector DataSet_ditie = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("PipeLine3D");
			DatasetVector dataset_PipePoint = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("PipePoint3D");

			Layer3DSettingVector setvetor = new Layer3DSettingVector();
			setvetor.getStyle()
					.setAltitudeMode(AltitudeMode.RELATIVE_TO_GROUND);
			setvetor.getStyle().setLineSymbolID(962048);
			setvetor.getStyle().setLineWidth(0.35);

			Layer3DDataset layer_ditie = m_sceneControl.getScene().getLayers()
					.add(DataSet_ditie, setvetor, true, "PipeLine3D");
			Layer3DDataset layer_pipepoint = m_sceneControl.getScene()
					.getLayers().add(dataset_PipePoint, setvetor, true,
							"PipePoint3D");

			if (dataset_PipePoint != null) {
				
                Theme3DUnique theme = new Theme3DUnique();
				theme.setUniqueExpression("SmID");

				GeoStyle3D style3D = new GeoStyle3D();
				style3D.setAltitudeMode(AltitudeMode.RELATIVE_TO_GROUND);
				style3D.setMarker3DScaleX(2.2);
				style3D.setMarker3DScaleY(2.2);
				style3D.setMarker3DScaleZ(2.2);
				style3D.setMarker3D(true);

				theme.setDefaultStyle(style3D);
				Recordset rs = dataset_PipePoint.query("", CursorType.STATIC);
				rs.moveFirst();
				int i = 1;

				while (!rs.isEOF()) {
					Theme3DUniqueItem item = getUniqueItem(rs,style3D);
					theme.add(item);
					rs.moveNext();
				}
				rs.dispose();
				Layer3DDataset layer_theme = (Layer3DDataset)m_sceneControl.getScene().getLayers()
						.add(dataset_PipePoint, theme, true, "underPipeLine3D");
				layer_theme.setSelectable(false);
				Layer3DSettingVector setting = (Layer3DSettingVector)layer_theme.getAdditionalSetting();
				setting.getStyle().setAltitudeMode(com.supermap.data.AltitudeMode.RELATIVE_TO_GROUND);
				layer_theme.updateData();
				
				if (m_sceneControl.getScene().getLayers().contains(
						"PipePoint3D")) {
					m_sceneControl.getScene().getLayers().remove("PipePoint3D");
				}
				Camera camera = new Camera(116.389952,39.992202, 2.236184);
				camera.setTilt(66.182019);
				camera.setHeading(306.425696);
				m_sceneControl.getScene().fly(camera, 5);

				JTextField text = new JTextField();
				text.requestFocus();
			}
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	private Theme3DUniqueItem getUniqueItem(Recordset rs,GeoStyle3D style3D){
		Theme3DUniqueItem item = new Theme3DUniqueItem();
		item.setUnique(String.valueOf(rs.getID()));
		GeoStyle3D tempStyle = new GeoStyle3D(style3D);
		pickNodeToLine(rs.getID());
		GeoPoint3D geoPoint3D = (GeoPoint3D)rs.getGeometry();

        ArrayList<Vector3d> arrVector3d = new ArrayList<Vector3d>();
        for (int curIndex = 0; curIndex < m_SelectedGeoNetwork.size(); curIndex++)
        {
            NetworkGeometry geoNetWork = m_SelectedGeoNetwork.get(curIndex);

            if (geoNetWork != null)
            {
                Vector3d vec = new Vector3d();
                GeoLine3D geoLine = (GeoLine3D)geoNetWork.geometry;
                if (geoNetWork.bIsIn)
                {
                    if (geoLine != null)
                    {
                        Point3Ds point3D = geoLine.getPart(0);
                        Point3D pntTNode = point3D.getItem(1);
                        Point3D pntFNode = point3D.getItem(0);
                        Vector3d vecTNode = Vector3d.sphericalToCartesian(pntTNode.getX() * DTOR, pntTNode.getY() * DTOR, pntTNode.getZ());
                        Vector3d vecFNode = Vector3d.sphericalToCartesian(pntFNode.getX() * DTOR, pntFNode.getY() * DTOR, pntFNode.getZ());

                        vec = new Vector3d(vecFNode.x - vecTNode.x, vecFNode.y - vecTNode.y, vecFNode.z - vecTNode.z);
                        vec.Normalize();
                    }
                }
                else
                {
                    if (geoLine != null)
                    {
                        Point3Ds point3D = geoLine.getPart(0);
                        Point3D pntFNode = point3D.getItem(0);
                        Point3D pntTNode = point3D.getItem(1);
                        Vector3d vecTNode = Vector3d.sphericalToCartesian(pntTNode.getX() * DTOR, pntTNode.getY() * DTOR, pntTNode.getZ());
                        Vector3d vecFNode = Vector3d.sphericalToCartesian(pntFNode.getX() * DTOR, pntFNode.getY() * DTOR, pntFNode.getZ());

                        vec = new Vector3d(vecTNode.x - vecFNode.x, vecTNode.y - vecFNode.y, vecTNode.z - vecFNode.z);
                        vec.Normalize();
                    }
                }

                vec = Vector3d.fromAngleAxis(vec, -geoPoint3D.getX() * DTOR, new Vector3d(0, 1, 0));
                vec = Vector3d.fromAngleAxis(vec, geoPoint3D.getY() * DTOR, new Vector3d(1, 0, 0));
                vec.Normalize();
                arrVector3d.add(vec);
            }
        }
		Vector3d vecX = new Vector3d();
		Vector3d vecY = new Vector3d();
		Vector3d vecZ = new Vector3d();
		switch (m_SelectedGeoNetwork.size()) {
		// 弯头或阀门
		// Valve
		case 2: 
		{
			if (arrVector3d.size() == 2) {
				vecX = arrVector3d.get(0).Normalize().Sub();
				vecY = arrVector3d.get(1).Normalize();
			}

			double dAngle1_2 = vecX.DotProduct(vecY);
			if (Math.abs(dAngle1_2) < 0.3) {
				tempStyle.setMarkerSymbolID(54441);
			} else if (-1 < dAngle1_2 && dAngle1_2 < 0.7) {
				tempStyle.setMarkerSymbolID(54442);
			} else {
				vecY.z = 0;
				Vector3d vec = new Vector3d(0, 0, 1);
				vecX = vecY.CrossProduct(vec);
				tempStyle.setMarkerSymbolID(54435);
			}
		}
			break;
			// 三通
			// Three go through
		case 3: 
		{
			tempStyle.setMarkerSymbolID(54437);
			if (arrVector3d.size() == 3) {
				Vector3d vec1 = arrVector3d.get(0);
				Vector3d vec2 = arrVector3d.get(1);
				Vector3d vec3 = arrVector3d.get(2);

				double dAngle1_2 = vec1.DotProduct(vec2);
				double dAngle1_3 = vec1.DotProduct(vec3);
				double dAngle2_3 = vec2.DotProduct(vec3);
				if (Math.abs(dAngle1_2) < 0.3) {
					if (Math.abs(dAngle1_3) < 0.3) {
						vecX = vec1.Normalize().Sub();
						vecY = vec2.Normalize();
					} else {
						vecX = vec2.Normalize().Sub();
						vecY = vec1.Normalize();
					}
				} else {
					vecX = vec3.Normalize().Sub();
					vecY = vec1.Normalize();
				}
			}
		}
			break;
			// 四通
			// Four go through
		case 4: 
		{
			tempStyle.setMarkerSymbolID(54438);
			if (arrVector3d.size() == 4) {
				Vector3d vec1 = arrVector3d.get(0);
				Vector3d vec2 = arrVector3d.get(1);
				Vector3d vec3 = arrVector3d.get(2);

				double dAngle1_2 = vec1.DotProduct(vec2);
				double dAngle1_3 = vec1.DotProduct(vec3);

				vecX = vec1;
				if (Math.abs(dAngle1_2) < 0.3) {
					vecY = vec2.Normalize();
				} else {
					vecY = vec3.Normalize();
				}
			}
		}
			break;
			// 结点
			// Vertex
		default: 
		{
			tempStyle.setMarkerSymbolID(54440);
		}
			break;
		}
		
		if(m_SelectedGeoNetwork.size() > 1)
		{
			vecZ = vecX.CrossProduct(vecY);
			vecZ.Normalize();
			
			Vector3d vecRotate = Vector3d.toEulerAnglesXYZ(vecX, vecY, vecZ);
			tempStyle.setMarker3DRotateX(-vecRotate.x * RTOD + 360);
			tempStyle.setMarker3DRotateY(-vecRotate.y * RTOD + 360);
			tempStyle.setMarker3DRotateZ(-vecRotate.z * RTOD + 360);
			
		}
		
		item.setStyle(tempStyle);
		return item;
	}

	private int pickNodeToLine(int nID) {
		int nCount = 0;
		try {
			m_SelectedGeoNetwork.clear();
			DatasetVector DvLine = (DatasetVector) m_workspace.getDatasources()
					.get(0).getDatasets().get("PipeLine3D");
			DatasetVector DvNode = (DatasetVector) m_workspace.getDatasources()
					.get(0).getDatasets().get("PipePoint3D");

			int[] id = { nID };
			Recordset recordset = DvNode.query(id, CursorType.STATIC);
			Geometry pGeoNode = recordset.getGeometry();

			String strNodeID = "PtID";
			Integer nNodeID = Integer.valueOf(recordset
					.getFieldValue(strNodeID).toString());
			String strFNode = "FNode";
			String strTNode = "ENode";
			QueryParameter queryparameter = new QueryParameter();
			queryparameter.setCursorType(CursorType.STATIC);
			queryparameter
					.setAttributeFilter(String.format("%s=%s or %s=%s",
							strFNode, nNodeID, strTNode, nNodeID));
			Recordset Rs = DvLine.query(queryparameter);
			nCount = Rs.getRecordCount();
			Rs.moveFirst();
			while (!Rs.isEOF()) {
				GeoLine3D geoline3D = (GeoLine3D) Rs.getGeometry();
				NetworkGeometry NetWorkGeometry = new NetworkGeometry();
				NetWorkGeometry.nID = geoline3D.getID();
				NetWorkGeometry.nFNode = Integer.valueOf(Rs.getFieldValue(
						strFNode).toString());
				NetWorkGeometry.nTNode = Integer.valueOf(Rs.getFieldValue(
						strTNode).toString());

				if (!m_SelectedGeoNetwork.contains(NetWorkGeometry)) {
					if (NetWorkGeometry.nTNode == nNodeID) {
						NetWorkGeometry.geometry = geoline3D;
						m_SelectedGeoNetwork.add(NetWorkGeometry);// 流入对象在第一位
					} else {
						NetWorkGeometry.bIsIn = false;
						NetWorkGeometry.geometry = geoline3D;
						m_SelectedGeoNetwork.add(NetWorkGeometry);// 线对象
					}
				}

				Rs.moveNext();
			}
			Rs.dispose();

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
		return nCount;
	}

	/*
	 * 加载北京地铁数据
	 * Add Bejing railway data
	 */
	private void loadSubway() {
		try {
			if (!m_sceneControl.getScene().getLayers().contains("ditie3")) {
				// 加载北京地铁数据集
				// Add Bejing railway dataset
				JFrame frame = new JFrame();
				DatasetVector DataSet_ditie = (DatasetVector) m_workspace
						.getDatasources().get(0).getDatasets().get(
								"BeijingSubway");
				DatasetVector dataset_label = (DatasetVector) m_workspace
						.getDatasources().get(0).getDatasets().get("StopsName");
				Layer3DSettingVector setvetor = new Layer3DSettingVector();
				setvetor.getStyle().setAltitudeMode(
						AltitudeMode.RELATIVE_TO_GROUND);
				setvetor.getStyle().setBottomAltitude(5);
				setvetor.getStyle().setLineWidth(100);
				setvetor.getStyle().setLineSymbolID(962046);
				setvetor.getStyle().setMarkerColor(Color.WHITE);
				setvetor.getStyle().setMarkerSize(1);
				setvetor.getStyle().setMarkerScale(1.5);
				setvetor.getStyle().setMarkerFile("../../SampleData/Symbol3DDisplay/SubwayMarkerLibrary.png");
				setvetor.getStyle().setMarkerSizeFixed(true);

                
				Layer3DDataset layer_ditie = m_sceneControl.getScene()
						.getLayers().add(DataSet_ditie, setvetor, true,
								"ditie3");

				// 标签专题图
				// Label thematic mapping
				Theme3DLabel themeLabel = new Theme3DLabel();
				if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
				{
					themeLabel.setLabelExpression("Name");
				}
				else
				{
					themeLabel.setLabelExpression("Name_en");
				}
				themeLabel.getUniformStyle().setSizeFixed(true);
				themeLabel.getUniformStyle().setForeColor(Color.YELLOW);
				themeLabel.getUniformStyle().setBackColor(Color.BLACK);
				themeLabel.getUniformStyle().setOutline(true);
				themeLabel.getUniformStyle().setFontScale(1);
				themeLabel.getUniformStyle().setWeight(7);
				themeLabel.getUniformStyle().setFontScale(1);
				themeLabel.getUniformStyle().setSizeFixed(true);
				themeLabel.getUniformStyle().setAlignment(TextAlignment.MIDDLERIGHT);

				Layer3D layer1 = m_sceneControl.getScene().getLayers().add(
						dataset_label, setvetor, true);
				Layer3D layer_label = m_sceneControl.getScene().getLayers()
						.add(dataset_label, themeLabel, true);
				layer_label.setAlwaysRender(true);

				Camera camera = m_sceneControl.getScene().getCamera();
				camera.setAltitudeMode(AltitudeMode.RELATIVE_TO_GROUND);
				camera.setAltitude(5508.4005263671279);
				camera.setHeading(0.016679026847733185);
				camera.setLatitude(39.908042501262457);
				camera.setLongitude(116.39138653936693);
				camera.setTilt(45.45877650025642);
				m_sceneControl.getScene().fly(camera, 2);
			}

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

	}

	/*
	 * 加载数据集
	 * Add dataset
	 */
	public void loadDataset() {
		try {
			WorkspaceConnectionInfo conInfo = new WorkspaceConnectionInfo(
					"../../SampleData/Symbol3DDisplay/Symbol3DDisplay.sxwu");
			conInfo.setType(WorkspaceType.SXWU);
			m_workspace.open(conInfo);
			
			//m_sceneControl.getScene().setWorkspace(m_workspace);

            m_sceneControl.getScene().open("OlympicGreen");

			// 导入三维符号库文件
            // Import the 3D symbol library
			Resources resource = m_workspace.getResources();
			SymbolLineLibrary linelibrary = resource.getLineLibrary();
			linelibrary
					.fromFile("../../SampleData/SymbolLibrary/Library3D/Line3DLibrary.lsl");
			SymbolMarkerLibrary markerlibrary = resource.getMarkerLibrary();
			markerlibrary
					.fromFile("../../SampleData/SymbolLibrary/Library3D/Marker3DLibrary.sym");

			// 画线
			// Draw line
			m_sceneControl.addTrackedListener(m_tracked3DListener);
			// 画点
			// Draw point
			m_sceneControl.addMouseListener(m_mouseClick);

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

	}

	/*
	 * 地铁八号线动态飞行
	 * Simulate Subway Line Running
	 */
	public void flyUndergroud() {
		try {
			
			if (!m_sceneControl.getScene().getLayers().contains("Ground@OlympicGreen")) {
				loadOlympicGreen();
			}
			
			// 加载飞行路线所需的数据和风格
			// Add data and style related to flying path
			m_sceneControl.getScene().setCameraFOV(60);
			// “北京地铁”和"OlympicGreen"数据集已经加载过，这里不需再添加
			// "Beijing Subway" and "OlympicGreen" have been already added
			// 为“北京地铁”设置风格
			// Set styles for Beijing Railway
			DatasetVector dataSet_ditie = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("BeijingSubway");
			Layer3DSettingVector settingvetor = new Layer3DSettingVector();
			settingvetor.getStyle().setAltitudeMode(
					AltitudeMode.RELATIVE_TO_GROUND);
			settingvetor.getStyle().setLineWidth(3);
			settingvetor.getStyle().setLineSymbolID(962046);
			settingvetor.getStyle().setBottomAltitude(-24);
			Layer3DDataset layer_ditie1 = m_sceneControl.getScene().getLayers()
					.add(dataSet_ditie, settingvetor, true, "ditie");
			layer_ditie1.setSelectable(false);

			// Load "ditie_line8"数据集作为地铁八号线的站点
			// Add "Subway_Line8" dataset as subway stations
			DatasetVector DataSet_ditieLine8 = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get("Subway_Line8");
			Layer3DSettingVector setvetor_ditieLine8 = new Layer3DSettingVector();

			setvetor_ditieLine8.setStyle(new GeoStyle3D());
			setvetor_ditieLine8.getStyle().setAltitudeMode(
					AltitudeMode.RELATIVE_TO_GROUND);
			setvetor_ditieLine8.getStyle().setBottomAltitude(10);
			Layer3DDataset layer_ditieLine8 = m_sceneControl.getScene()
					.getLayers().add(DataSet_ditieLine8, setvetor_ditieLine8,
							true, "地铁八号线");
			layer_ditieLine8.updateData();

			// Load"北京地铁L_2"数据集作为地铁巷道
			// Load"SubwayL_2" dataset as roadway
			DatasetVector DataSet_ditieL_2 = (DatasetVector) m_workspace
					.getDatasources().get(0).getDatasets().get(
							"BeijingSubway_Tunnel");
			Layer3DSettingVector setvetor_ditieL_2 = new Layer3DSettingVector();
			setvetor_ditieL_2.getStyle().setAltitudeMode(
					AltitudeMode.RELATIVE_TO_GROUND);
			setvetor_ditieL_2.getStyle().setBottomAltitude(-26.5);
			setvetor_ditieL_2.getStyle().setLineSymbolID(962047);
			setvetor_ditieL_2.getStyle().setLineWidth(10);
			Layer3DDataset layer_ditieL_2 = m_sceneControl.getScene()
					.getLayers().add(DataSet_ditieL_2, setvetor_ditieL_2, true,
							"地铁巷道");
			layer_ditieL_2.updateData();

			// 加载北京地铁动画模型KML文件到场景
			// Add animation to the scene
			String KMLPath = "../../SampleData/Symbol3DDisplay/Model/Subway.kml";
			Layer3DKML layer_kml = (Layer3DKML) m_sceneControl.getScene()
					.getLayers().add(KMLPath, Layer3DType.KML, true, "模型动画");
			Feature3D geofeature = layer_kml.getFeatures().findFeature(1,
					Feature3DSearchOption.ALLFEATURES);
			GeoPlacemark geoplacemark = (GeoPlacemark) geofeature.getGeometry();
			GeoModel geomodel = (GeoModel) geoplacemark.getGeometry();
			geomodel.getAnimation().setPlayMode(PlayMode.ONCE);
			layer_kml.updateData();

			// 导入飞行路线文件
			// Import the flying path file
			Boolean i = flyManager.getRoutes().fromFile(
					"../../SampleData/Symbol3DDisplay/flyground.fpf");
			flyManager.getRoutes().setCurrentRouteIndex(0);
			flyManager.play();

			// 当飞行结束后触发事件
			// Trigger this event after finished flying
			m_sceneControl.getScene().refresh();
			// m_sceneControl.update(null);
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	private void loadOlympicGreen() {
		try {
			m_sceneControl.getScene().open("OlympicGreen");
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/*
	 * 动态飞行结束
	 * End flying
	 */
	public void flyUndergroudStop() {
		try {
			flyManager.stop();
			flyManager.getRoutes().remove(0);
			m_sceneControl.getScene().getUnderground().setVisible(false);
			m_sceneControl.getScene().getLayers().clear();
			loadOlympicGreen();

			// 初始化 camera
			// Initialize camera
			Camera camera_Olympic = new Camera(116.38435249933515,
					39.991643985278039, 128.56367969792336);
			camera_Olympic.setTilt(65.822683203756768);
			camera_Olympic.setHeading(26.092343265414662);
			m_sceneControl.getScene().fly(camera_Olympic, 2);

		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

	}

}

